package doublePEP;

/**This DoubleNode subclass defines the operations of DoubleEpisode nodes.*/
public class DoubleEpisode extends DoubleNode{
	/**error tag*/       private boolean isErrorTagged;
	/**search strength*/ private double searchStrength;
	/**response time*/   private double rt;

	/**Creates a new DoubleEpisode.*/
	public DoubleEpisode(){
		super( DoubleRunTime.nResponses );
		threshold = .02;
		isErrorTagged = false;
		searchStrength = 6;
	}

	/**Updates activation for each cycle and seraches responses.*/
	public void activate( int modelNumber ){
		update( ) ;
		if( isErrorTagged ) searchStrength = .02 ;
		if( activation > threshold ) respSearch( modelNumber ) ;
	}

	/**Activates DoubleResponse nodes and weakens connections.*/
	public void respSearch( int modelNumber ){
		double temp = (activation-threshold)*searchStrength ;
		if ( modelNumber == 1 ) {
			for( int j = 0 ; j < DoubleRunTime.nResponses ; j++ ){
				double output = temp*connections[j] ;
				DoubleRunTime.responses_1[j].retrieved += output ;
				DoubleResponse.totRetrieved += output ;
				connections[ j ] *= ( 1 - ( temp * decayConnect ) ) ;
			}
		} else {
			for(int j=0; j<DoubleRunTime.nResponses; j++){
				double output = temp*connections[j];
				DoubleRunTime.responses_2[j].retrieved += output;
				DoubleResponse.totRetrieved += output;
				connections[ j ] *= ( 1 - ( temp * decayConnect ) ) ;
			}
		}
	}

	/**Connects DoubleEpisode to activated DoubleResponse nodes.*/
	public void connect( int modelNumber ){
		for(int j=0; j<DoubleRunTime.nResponses; j++){
			connections[j] *= (1-writeConnect);
			if ( modelNumber == 1 ) {
				connections[j] += DoubleRunTime.responses_1[j].activation*writeConnect;
			} else {
				connections[j] += DoubleRunTime.responses_2[j].activation*writeConnect;
			}
		}
	}

	/**Tags a DoubleEpisode as an error.*/
	public void tagError(){
		isErrorTagged = true;
	}

	/**Updates activation for the newest episode.*/
	public void focus(){
		input += .05;
		update();
	}

	/**Records response time.
	 * @param respTime Response time.*/
	public void setRT(double respTime){
		rt = respTime;
	}

	/**Retrieves response time.
	 * @param cyc Cycle number.
	 * @param connect Connection strength.
	 * @return The current prediction strength of node.*/
	public double getRT(int cyc,double connect){
		double out = (1 - (range(((double)Math.pow(cyc-rt,2))/10000)))*connect;
		return out;
	}
}