/**Two parallel instantiation of the Parallel Episodic Processing (PEP) Model
 * Version 1.2.0
 * James R. Schmidt, Universiteit Gent
 * Fonds Wetenschappelijk Onderzoek - Vlaanderen (FWO)*/

package doublePEP;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class DoubleMain{
	//----------VARIABLES------------------------------------------
	/**Data sources for model 1 and model 2*/
	private static String source1 = new String( "inFile_high_low_mixed.txt" ) ;
	private static String source2 = new String( "inFile_low_high_mixed.txt" ) ;
	private static String s1_rand = new String( "s1_randomized.txt" ) ;
	private static String s2_rand = new String( "s2_randomized.txt" ) ;
	
	public static double influence = .25 ;
	/**number of subjects*/                static int numSubjects = 5000 ;
	/**number of trials (no replacement)*/ static int numSamples = 336 ;
	/**number of responses*/               static int numResponses = 4;
	/**number of targets*/                 static int numTargets = 4;
	/**target bias*/                       static double targBias = .9;
	/**target signal*/                     static double targSignal = .9;
	/**number of distracters per set*/     static int[] numDistracters = {4};
	/**distracter bias*/                   static double[] distBias = {.9};
	/**distracter signal*/                 static double[] distSignal = {.9};
	/**Set 1 distracter conflict*/         static boolean isConflict = true;
	/**number of conditons per variable*/  static int[] numPerVar = {2,2,2};
	//-------------------------------------------------------------

	/**For each participant, the main method runs the models
	 * with DoubleRunTime (the guts of the model(s)).*/
	public static void main(String[] args){
		// Matching Condition
		// Counter-balancing condition 1
		String s1 = source1 ;
		String s2 = source1 ;
		for(int y=0; y<numSubjects; y++){
			new DoubleDataSet( s1, s2 ) ;
			new DoubleRunTime( y + 1, s1, s2, s1_rand, s2_rand ) ;
			System.out.println(""+(y+1)+" of "+numSubjects+" subjects complete...");
		}
		// Counter-balancing condition 2
		s1 = source2 ;
		s2 = source2 ;
		for(int y=0; y<numSubjects; y++){
			new DoubleDataSet( s1, s2 ) ;
			new DoubleRunTime( y + 1, s1, s2, s1_rand, s2_rand ) ;
			System.out.println(""+(y+1)+" of "+numSubjects+" subjects complete...");
		}

		// Contrasting Condition
		// Counter-balancing condition 1
		s1 = source1 ;
		s2 = source2 ;
		for(int y=0; y<numSubjects; y++){
			new DoubleDataSet( s1, s2 ) ;
			new DoubleRunTime( y + 1, s1, s2, s1_rand, s2_rand ) ;
			System.out.println(""+(y+1)+" of "+numSubjects+" subjects complete...");
		}
		//Counter-balancing condition 2
		s1 = source2 ;
		s2 = source1 ;
		for(int y=0; y<numSubjects; y++){
			new DoubleDataSet( s1, s2 ) ;
			new DoubleRunTime( y + 1, s1, s2, s1_rand, s2_rand ) ;
			System.out.println(""+(y+1)+" of "+numSubjects+" subjects complete...");
		}
	}
}