package doublePEP;

/**This superclass defines the basic fields and methods of DoubleNodes*/
public class DoubleNode{
	/**rate of decay*/                   public double decay;
	/**incoming activation*/             public double input;
	/**current activation*/              public double activation;
	/**threshold to pass on activation*/ public double threshold;
	/**search strength*/                 public double strength;
	/**rate of connection building*/     public double writeConnect;
	/**rate of decay for connections*/   public double decayConnect;
	/**array of connections*/            public double[] connections;
	/**number of connections of model 1*/public static int nConnections_1 = 0;
	/**number of connections of model 2*/public static int nConnections_2 = 0;
	/**target truth value*/              public boolean isTarget;

	/**Creates a new DoubleNode.*/
	public DoubleNode(){
		decay = .01;
		input = 0;
		activation = 0;
		isTarget = false;
	}

	/**Creates a new DoubleNode with connections to other nodes.
	 * @param connectNum Number of possible connections to other nodes.*/
	public DoubleNode(int connectNum){
		this();
		decayConnect = .0001;
		writeConnect = .01;
		connections = new double[connectNum];
	}

	/**Updates activation and searches memory.*/
	public void activate( int modelNumber ){
		update();
		if(activation>threshold) memSearch( modelNumber ) ;
		if(DoubleRunTime.isStimuli && !isTarget) memConnect( modelNumber ) ;
	}

	/**Uses input to update the activation of the DoubleNode.*/
	public void update(){
		input = range(input);
		activation = activation*(1-decay) + input*decay;
		activation = range(activation);
		input = 0;
	}

	/**Ranges activation between 0 and 1.
	 * @param in Value to be ranged.
	 * @return The newly ranged value.*/
	public double range(double in){
		if(in<0) return 0;
		else if(in>1) return 1;
		else return in;
	}

	/**Ranges activation between 0 and max.
	 * @param in Value to be ranged.
	 * @param max Maximum value to return.
	 * @return The newly ranged value.*/
	public double range(double in,double max){
		if(in<0) return 0;
		else if(in>max) return max;
		else return in;
	}

	/**Activates episodes and weakens connections.*/
	public void memSearch( int modelNumber ){
		int number ;
		double temp = (activation-threshold)*strength;
		if ( modelNumber == 1 ) {
			number = nConnections_1 ;
			for(int j=0; j<number; j++){
				DoubleRunTime.memory_1[j].input += temp*connections[j];
				connections[j] *= (1-(temp*decayConnect));
			}
		} else {
			number = nConnections_2 ;
			for(int j=0; j<number; j++){
				DoubleRunTime.memory_2[j].input += temp*connections[j];
				connections[j] *= (1-(temp*decayConnect));
			}
		}
	}

	/**Builds a connection to new episode.*/
	public void memConnect( int modelNumber ){
		int number ;
		double temp = range(activation-threshold)*strength;
		if ( modelNumber == 1 ) {
			number = nConnections_1 ;
			DoubleRunTime.memory_1[ number ].input += temp*connections[ number ];
		} else {
			number = nConnections_2 ;
			DoubleRunTime.memory_2[ number ].input += temp*connections[ number ];
		}
		connections[ number ] *= (1-writeConnect);
		connections[ number ] += temp*writeConnect;
	}

	/**Sets memory search strength for target.*/
	public void targeted(){
		strength = 0;
		isTarget = true;
	}

	/**Sets memory search strength for distracter.
	 * @param nSets The number of distracter sets.*/
	public void ignored(int nSets){
		strength = 6/(double)nSets ;
		isTarget = false;
	}
}