package doublePEP;

import java.io.*;
import java.util.*;

/**This class is guts of the model, where each run is performed.*/
public class DoubleRunTime{
	/**number of targets*/          static int nTargets = DoubleMain.numTargets;
	/**number of distracter sets*/  static int nSets = DoubleMain.numDistracters.length;
	/**number of distracters*/      static int[] nDistract = DoubleMain.numDistracters;
	/**number of responses*/        static int nResponses = DoubleMain.numResponses;
	/**set 1 conflict*/             static boolean isConflict = DoubleMain.isConflict;
	/**number of variables*/        static int nVariables = DoubleMain.numPerVar.length;
	/**target signal strength*/     static double tSignal = DoubleMain.targSignal;
	/**distracter signal strength*/ static double[] dSignal = DoubleMain.distSignal;
	/**target processing bias*/     static double tBias = DoubleMain.targBias;
	/**distracter processing bias*/ static double[] dBias = DoubleMain.distBias;

	/**number of episodes for model 1*/
	static int nEpisodes_1 = 3000 ; //should be greater than number of trials
	/**episodic memory for model 1*/
	static DoubleEpisode[] memory_1 = new DoubleEpisode[nEpisodes_1] ;
	/**episode counter for model 1*/
	static int eNumber_1 ;
	/**target DoubleInput nodes for model 1*/
	static DoubleInput[] targets_1 = new DoubleInput[nTargets];
	/**distracter DoubleInput nodes for model 1*/
	static DoubleInput[][] distracters_1 = new DoubleInput[nSets][];
	/**DoubleIdentity nodes for model 1*/
	static DoubleIdentity[] identities_1 = new DoubleIdentity[nTargets];
	/**DoubleResponse nodes for model 1*/
	static DoubleResponse[] responses_1 = new DoubleResponse[nResponses];

	/**number of episodes for model 2*/
	static int nEpisodes_2 = 3000 ; //should be greater than number of trials
	/**episodic memory for model 2*/
	static DoubleEpisode[] memory_2 = new DoubleEpisode[nEpisodes_2] ;
	/**episode counter for model 2*/
	static int eNumber_2 ;
	/**target DoubleInput nodes for model 2*/
	static DoubleInput[] targets_2 = new DoubleInput[nTargets];
	/**distracter DoubleInput nodes for model 2*/
	static DoubleInput[][] distracters_2 = new DoubleInput[nSets][];
	/**DoubleIdentity nodes for model 2*/
	static DoubleIdentity[] identities_2 = new DoubleIdentity[nTargets];
	/**DoubleResponse nodes for model 2*/
	static DoubleResponse[] responses_2 = new DoubleResponse[nResponses];

	/**Task/Model sequence*/
	static Integer[ ] whichModelTest = new Integer[ 32 ] ;
	static Integer[ ] whichModel = new Integer[ DoubleMain.numSamples * 2 ] ;

	/**end of trial marker*/  static boolean isTrialOver;
	/**condition number*/     static int[] condition = new int[nVariables];
	/**target number*/        static int target;
	/**distrater number*/     static int[] distracter = new int[nSets];
	/**cycle number*/         static int cycle;
	/**correct response*/     static int correctResp;
	/**actual response*/      static int response;
	/**accuracy*/             static int accuracy;
	/**stimulus on screen?*/  static boolean isStimuli = true;

	/**Defines the nodes of the model defined and runs the trials.
	 * @param subj The subject number.*/
	DoubleRunTime( int subj, String source1, String source2, String source3, String source4 ) {

		//Creates task/model sequence
		// test block (32 trials)
		Arrays.fill( whichModelTest, 0, 16, 1 ) ; 
		Arrays.fill( whichModelTest, 16, 32, 2 ) ;
		List<Integer> dummy = Arrays.asList( whichModelTest ) ;
		Collections.shuffle( dummy ) ;
		dummy.toArray( whichModelTest ) ;
		
		System.arraycopy( whichModelTest, 0, whichModel, 0, whichModelTest.length ) ;

		// merge with 10 experimental blocks (640 trials)
		Integer[ ] dummyBlock = new Integer[ 64 ] ;
		for ( int b = 0; b < 10; b++ ) {
			Arrays.fill( dummyBlock, 0, 32, 1 ) ; 
			Arrays.fill( dummyBlock, 32, 64, 2 ) ;
			List<Integer> dummyList = Arrays.asList( dummyBlock ) ;
			Collections.shuffle( dummyList ) ;
			dummyList.toArray( dummyBlock ) ;
			System.arraycopy( dummyBlock, 0, whichModel, 32 + b * 64, 64 ) ;
		}
		
		//sets up the nodes for the distracter sets of model 1
		for(int set=0; set<nSets; set++){
			distracters_1[set] = new DoubleInput[nDistract[set]];
			for(int k=0; k<nDistract[set]; k++){
				distracters_1[set][k] = new DoubleInput( nEpisodes_1 ) ;
				distracters_1[set][k].ignored(nSets);
			}
		}

		//sets up the nodes for the distracter sets of model 2
		for(int set=0; set<nSets; set++){
			distracters_2[set] = new DoubleInput[nDistract[set]];
			for(int k=0; k<nDistract[set]; k++){
				distracters_2[set][k] = new DoubleInput( nEpisodes_2 );
				distracters_2[set][k].ignored( nSets );
			}
		}

		//sets up the rest of the nodes of model 1
		for(int i=0; i<nEpisodes_1; i++) memory_1[i] = new DoubleEpisode();
		for(int i=0; i<nTargets; i++){
			targets_1[i] = new DoubleInput( nEpisodes_1 );
			targets_1[i].targeted();
			identities_1[i] = new DoubleIdentity();
		}
		for(int i=0; i<nResponses; i++) responses_1[i] = new DoubleResponse();

		eNumber_1 = 0 ;
		DoubleNode.nConnections_1 = eNumber_1 ;

		//sets up the rest of the nodes of model 2
		for(int i=0; i<nEpisodes_2; i++) memory_2[i] = new DoubleEpisode();
		for(int i=0; i<nTargets; i++){
			targets_2[i] = new DoubleInput( nEpisodes_2 );
			targets_2[i].targeted();
			identities_2[i] = new DoubleIdentity();
		}
		for(int i=0; i<nResponses; i++) responses_2[i] = new DoubleResponse();

		eNumber_2 = 0 ;
		DoubleNode.nConnections_2 = eNumber_2 ;

		//---Open Streams---
		StringTokenizer tokenizer_1 = null;
		BufferedReader in_1 = null;
		FileOutputStream out = null;
		PrintStream p = null;

		String[ ] source1Parts = source1.split("_") ;
		String[ ] source2Parts = source2.split("_") ;

		try{
			in_1 = new BufferedReader( new FileReader( source3 ) ) ;
			String outFile = new String( ) ;
			if ( source1Parts[ 1 ].equals( source2Parts[ 1 ] ) ) {
				outFile = ( "all_data" + "_" + "matching" + "_" + source1Parts[ 1 ] + source1Parts[ 2 ] + "_" + source2Parts[ 1 ] + source2Parts[ 2 ] + "_" + DoubleMain.numSubjects + "_" + DoubleMain.numSamples * 2 + "_" + DoubleMain.influence + ".txt" ) ;
			} else {
				outFile = ( "all_data" + "_" + "contrasting" + "_" + source1Parts[ 1 ] + source1Parts[ 2 ] + "_" + source2Parts[ 1 ] + source2Parts[ 2 ] + "_" + DoubleMain.numSubjects + "_" + DoubleMain.numSamples * 2 + "_" + DoubleMain.influence + ".txt" ) ;
			}
			if ( subj == 1 ) {
				out = new FileOutputStream( outFile ) ;
			} else {
				out = new FileOutputStream( outFile, true ) ;
			}
			p = new PrintStream(out);
			tokenizer_1 = new StringTokenizer(in_1.readLine()); //<--Trial 1
		}
		catch(Exception e){
			System.out.println("Error setting up input and output streams.");
		}

		//---Open Streams---
		StringTokenizer tokenizer_2 = null;
		BufferedReader in_2 = null;

		try{
			in_2 = new BufferedReader( new FileReader( source4 ) ) ;
			tokenizer_2 = new StringTokenizer(in_2.readLine( ) ) ; //<--Trial 1
		}
		catch(Exception e){
			System.out.println("Error setting up input and output streams.");
		}


		for ( int trial = 0 ; trial < ( DoubleMain.numSamples * 2 ) ; trial++ ) {

			int modelNumber = whichModel[ trial ] ;

			if ( modelNumber == 1 ) {

				//run trial for task/model 1

				//split up trial info into variables
				correctResp = Integer.parseInt(tokenizer_1.nextToken());
				target = Integer.parseInt(tokenizer_1.nextToken());
				for(int ns=0; ns<nSets; ns++)
					distracter[ns] = Integer.parseInt(tokenizer_1.nextToken());
				for(int nc=0; nc<nVariables; nc++)
					condition[nc] = Integer.parseInt(tokenizer_1.nextToken());
				if ( source1.equals( source2 ) ) {
					condition[ nVariables - 1 ] = 1 ;
				}
				response = 999; accuracy = 999;

				//---Define Inputs---
				//noise activation is added to each input node
				for(int i=0; i<nTargets; i++) targets_1[i].param(tBias,0);
				for(int j=0; j<nSets; j++)
					for(int i=0; i<nDistract[j]; i++) distracters_1[j][i].param(dBias[j],0);

				//the presented nodes are activated with noise plus a signal
				if(nTargets>0) targets_1[target].param(tBias,tSignal);
				for(int j=0; j<nSets; j++)
					if(nDistract[j]>0)
						distracters_1[j][distracter[j]].param(dBias[j],dSignal[j]);

				//---Run Cycles---
				isTrialOver = false;
				isStimuli = true;
				//run a cycle and monitor for colour and response identification
				for(cycle=1; isTrialOver==false; cycle++){
					go( modelNumber );
					if(monitorTarg( modelNumber ) ) if( monitorResp( modelNumber ) ) isTrialOver=true;
					if(cycle>1999) isTrialOver=true;
				}

				//---Post-Trial Coding---
				//calculate accuracy
				if(correctResp==response) accuracy = 1;
				else{
					accuracy = 0;
					memory_1[eNumber_1].tagError();
				}
				//write trial info to file
				p.print(subj+"\t"+response+"\t"+target+"\t");
				for(int k=0; k<nSets; k++) p.print(distracter[k]+"\t");
				for(int m=0; m<nVariables; m++) p.print(condition[m]+"\t");
				p.println(correctResp+"\t"+cycle+"\t"+accuracy+"\t"+modelNumber);
				//record response time into memory
				memory_1[eNumber_1].setRT(cycle);

				//---Run ITI---
				isStimuli = false;
				//null input cycles (inter-trial interval)
				for(int i=0; i<nTargets; i++) targets_1[i].param(0,0);
				for(int j=0; j<nSets; j++)
					for(int i=0; i<nDistract[j]; i++) distracters_1[j][i].param(0,0);

				//300 cycle ITI
				for(int i=0; i<300; i++){
					go( modelNumber );
				}

				//increase counters
				eNumber_1++;
				DoubleNode.nConnections_1 = eNumber_1;
				if(eNumber_1>=nEpisodes_1) System.out.println("Error: Not enough Episodes in Memory of Model 1.");

				//---Read Next Trial---
				try{
					//---For task/model 1---
					String tempTrial = new String(in_1.readLine());
					if(tempTrial.equals("END_OF_FILE")) in_1 = null;
					else tokenizer_1 = new StringTokenizer(tempTrial);
				}
				catch(Exception e2){
					System.out.println("No END_OF_FILE marker");
					in_1 = null;
				}

			} else {

				//run trial for task/model 2

				//split up trial info into variables
				correctResp = Integer.parseInt(tokenizer_2.nextToken());
				target = Integer.parseInt(tokenizer_2.nextToken());
				for(int ns=0; ns<nSets; ns++)
					distracter[ns] = Integer.parseInt(tokenizer_2.nextToken());
				for(int nc=0; nc<nVariables; nc++)
					condition[nc] = Integer.parseInt(tokenizer_2.nextToken());
				if ( source1.equals( source2 ) ) {
					condition[ nVariables - 1 ] = 1 ;
				}
				response = 999; accuracy = 999;

				//---Define Inputs---
				//noise activation is added to each input node
				for(int i=0; i<nTargets; i++) targets_2[i].param(tBias,0);
				for(int j=0; j<nSets; j++)
					for(int i=0; i<nDistract[j]; i++) distracters_2[j][i].param(dBias[j],0);

				//the presented nodes are activated with noise plus a signal
				if(nTargets>0) targets_2[target].param(tBias,tSignal);
				for(int j=0; j<nSets; j++)
					if(nDistract[j]>0)
						distracters_2[j][distracter[j]].param(dBias[j],dSignal[j]);

				//---Run Cycles---
				isTrialOver = false;
				isStimuli = true;
				//run a cycle and monitor for colour and response identification
				for(cycle=1; isTrialOver==false; cycle++){
					go( modelNumber );
					if(monitorTarg( modelNumber ) ) if( monitorResp( modelNumber ) ) isTrialOver=true;
					if(cycle>1999) isTrialOver=true;
				}

				//---Post-Trial Coding---
				//calculate accuracy
				if(correctResp==response) accuracy = 1;
				else{
					accuracy = 0;
					memory_2[eNumber_2].tagError();
				}
				//write trial info to file
				p.print(subj+"\t"+response+"\t"+target+"\t");
				for(int k=0; k<nSets; k++) p.print(distracter[k]+"\t");
				for(int m=0; m<nVariables; m++) p.print(condition[m]+"\t");
				p.println(correctResp+"\t"+cycle+"\t"+accuracy+"\t"+modelNumber);
				//record response time into memory
				memory_2[eNumber_2].setRT(cycle);

				//---Run ITI---
				isStimuli = false;
				//null input cycles (inter-trial interval)
				for(int i=0; i<nTargets; i++) targets_2[i].param(0,0);
				for(int j=0; j<nSets; j++)
					for(int i=0; i<nDistract[j]; i++) distracters_2[j][i].param(0,0);

				//300 cycle ITI
				for(int i=0; i<300; i++){
					go( modelNumber );
				}

				//increase counters
				eNumber_2++;
				DoubleNode.nConnections_2 = eNumber_2;
				if(eNumber_2>=nEpisodes_2) System.out.println("Error: Not enough Episodes in Memory of Model 2.");

				//---Read Next Trial---
				try{
					//---For task/model 1---
					String tempTrial = new String( in_2.readLine( ) ) ;
					if ( tempTrial.equals( "END_OF_FILE" ) ) in_2 = null ;
					else tokenizer_2 = new StringTokenizer( tempTrial ) ;
				}
				catch (Exception e2) {
					System.out.println( "No END_OF_FILE marker" ) ;
					in_2 = null ;
				}

			}

		}

		try{
			p.close( ) ; out.close( ) ;
		}
		catch( Exception e3 ){
			System.out.println( "Error closing input and output streams." ) ;
		}
	}


	//-------------------------------------------------------------------
	//------------------------------Methods------------------------------
	//-------------------------------------------------------------------

	/**Performs all the calculations for an individual cycle.*/
	static void go( int modelNumber ){

		if ( modelNumber == 1 ) {
			//---INPUTS---
			for(int j=0; j<nSets; j++){
				boolean isConf = isConflict;
				if(j>0) isConf=false; //only the first node set conflicts
				for(int i=0; i<nDistract[j]; i++) distracters_1[j][i].activate( modelNumber, i, isConf );
			}
			for(int i=0; i<nTargets; i++) targets_1[i].activate( modelNumber, i, isConflict );
			//---EPISODES---
			for(int i=0; i<eNumber_1; i++) memory_1[i].activate( modelNumber );
			memory_1[eNumber_1].focus();
			if(isStimuli) memory_1[eNumber_1].connect( modelNumber );
			//---IDENTITIES---
			for( int i = 0 ; i < nTargets ; i++ ) identities_1[ i ].activate( i, modelNumber ) ;
			DoubleIdentity.clear();
			//---RESPONSES---
			for ( int i = 0 ; i < nResponses ; i++ ) responses_1[ i ].activate( ) ;
			DoubleResponse.clear();
		} else {
			//---INPUTS---
			for(int j=0; j<nSets; j++){
				boolean isConf = isConflict;
				if(j>0) isConf=false; //only the first node set conflicts
				for(int i=0; i<nDistract[j]; i++) distracters_2[j][i].activate( modelNumber, i, isConf ) ;
			}
			for(int i=0; i<nTargets; i++) targets_2[i].activate( modelNumber, i, isConflict ) ;
			//---EPISODES---
			for(int i=0; i<eNumber_2; i++) memory_2[i].activate( modelNumber );
			memory_2[eNumber_2].focus();
			if(isStimuli) memory_2[eNumber_2].connect( modelNumber );
			//---IDENTITIES---
			for(int i=0; i<nTargets; i++) identities_2[i].activate( i, modelNumber );
			DoubleIdentity.clear();
			//---RESPONSES---
			for(int i=0; i<nResponses; i++) responses_2[i].activate();
			DoubleResponse.clear();
		}
	}

	/**Checks colour colour identification.
	 *@return If target is present.*/
	static boolean monitorTarg( int modelNumber ){
		boolean found = false;
		if ( modelNumber == 1 ) {
			for(int i=0; i<nTargets; i++)
				if(targets_1[i].activation>targets_1[i].threshold) found = true;
		} else {
			for(int i=0; i<nTargets; i++)
				if(targets_2[i].activation>targets_2[i].threshold) found = true;
		}
		return found;
	}

	/**Checks for temporal expectancy and generates a temporal bias for each model and merges them according to the influence of the respective other model defined in DoubleMain.
	 *@return Temporal expectancy bias.*/
	static double monitorTemp( int modelNumber ){
		double expect = 0;
		double expect_1 = 0 ;
		double expect_2 = 0 ;

		//Limit search on initial trials (to avoid runtime errors)
		int iLimit = 40; if(eNumber_1<40) iLimit=eNumber_1;
		//Search for temporal expectancies
		for(int i=1; i<=iLimit; i++)
			expect_1 += memory_1[eNumber_1-i].getRT(cycle,(Math.pow((40-i),2)*.00002));

		//Limit search on initial trials (to avoid runtime errors)
		if(eNumber_2<40) iLimit=eNumber_2;
		//Search for temporal expectancies
		for(int i=1; i<=iLimit; i++)
			expect_2 += memory_2[eNumber_2-i].getRT(cycle,(Math.pow((40-i),2)*.00002));

		if ( modelNumber == 1 ) { 
			expect = ( 1 - DoubleMain.influence ) * expect_1 + DoubleMain.influence * expect_2 ;
		} else {
			expect = ( 1 - DoubleMain.influence ) * expect_2 + DoubleMain.influence * expect_1 ;
		}

		expect -= .01; if(expect<0) expect=0; //small threshold
		if(expect>.15) expect=.15; //max expectancy drop
		return expect;
	}

	/**Checks for response identification and ends the trial if necessary.
	 *@return If the response has been found.*/
	static boolean monitorResp( int modelNumber ){
		boolean found = false;
		if ( modelNumber == 1 ) {
			for(int i=0; i<nResponses; i++){
				if(responses_1[i].activation>(responses_1[i].threshold-monitorTemp( modelNumber ) ) ) {
					found = true;
					//if two responses cross threshold at same time, response = 777
					if(response != 999) response = 777;
					else response = i;
				}
			}
		} else {
			for(int i=0; i<nResponses; i++){
				if(responses_2[i].activation>(responses_2[i].threshold-monitorTemp( modelNumber ) ) ) {
					found = true;
					//if two responses cross threshold at same time, response = 777
					if(response != 999) response = 777;
					else response = i;
				}
			}
		}
		return found;
	}
}